package com.gic.cloud.service.impl;

import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.cloud.dao.mapper.TabSysAccountGroupMapper;
import com.gic.cloud.dto.AccountGroupDTO;
import com.gic.cloud.entity.TabSysAccountGroup;
import com.gic.cloud.service.AccountGroupService;
import com.gic.commons.util.EntityUtil;

@Service("accountGroupService")
public class AccountGroupServiceImpl implements AccountGroupService{
    @Autowired
    private TabSysAccountGroupMapper tabSysAccountGroupMapper;

    @Override
    public Integer saveAccountGroup(AccountGroupDTO dto) {
        Date now = new Date();
        dto.setUpdateTime(now);
        dto.setCreateTime(now);
        dto.setStatus(1);
        TabSysAccountGroup record = EntityUtil.changeEntityNew(TabSysAccountGroup.class, dto);
        tabSysAccountGroupMapper.insert(record);
        return record.getAccountGroupId();
    }

    @Override
    public void editAccountGroup(AccountGroupDTO dto) {
        TabSysAccountGroup record = EntityUtil.changeEntityNew(TabSysAccountGroup.class, dto);
        record.setUpdateTime(new Date());
        tabSysAccountGroupMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public boolean isRepeatAccountGroupName(Integer enterpriseId, Integer accountGroupId, String accountGroupName) {
        int count = tabSysAccountGroupMapper.countByAccountGroupName(enterpriseId, accountGroupId, accountGroupName);
        if (count > 0) {
            return true;
        }
        return false;
    }

    @Override
    public int queryMaxSort(Integer enterpriseId) {
        return tabSysAccountGroupMapper.queryMaxSort(enterpriseId);
    }

    @Override
    public TabSysAccountGroup getAccountGroupById(Integer accountGroupId) {
        return tabSysAccountGroupMapper.selectByPrimaryKey(accountGroupId);
    }

    @Override
    public void sortAccountGroup(TabSysAccountGroup record, Integer targetSort) {
        List<TabSysAccountGroup> list = tabSysAccountGroupMapper.listAccountGroupOrderBySort(record.getEnterpriseId());
        Integer originalSort = record.getSort();
        if (originalSort > targetSort) {
            //向上拖拽
            for (int i = 0, length = list.size(); i < length; i++) {
                TabSysAccountGroup temp = list.get(i);
                //如果大于targetSort，都需要降低排序值,往后推
                boolean isNeedDown = temp.getSort() >= targetSort && temp.getSort() < originalSort;
                if (isNeedDown) {
                    updateSort(list.get(i + 1).getSort(), temp.getAccountGroupId());
                }
            }
        } else if (originalSort < targetSort) {
            //向下拖拽
            for (int i = 0, length = list.size(); i < length; i++) {
                TabSysAccountGroup temp = list.get(i);
                boolean isNeedUp = temp.getSort() <= targetSort && temp.getSort() > originalSort;
                if (isNeedUp) {
                    updateSort(list.get(i - 1).getSort(), temp.getAccountGroupId());
                }
            }
        }
        updateSort(targetSort, record.getAccountGroupId());
    }

    @Override
    public List<TabSysAccountGroup> listAccountGroupOrderBySort(Integer enterpriseId) {
        return tabSysAccountGroupMapper.listAccountGroupOrderBySort(enterpriseId);
    }

    @Override
    public void deleteAccountGroup(Integer accountGroupId) {
        tabSysAccountGroupMapper.deleteAccountGroup(accountGroupId);
    }

    private void updateSort(Integer sort, Integer accountGroupId) {
        TabSysAccountGroup record = new TabSysAccountGroup();
        record.setAccountGroupId(accountGroupId);
        record.setSort(sort);
        tabSysAccountGroupMapper.updateByPrimaryKeySelective(record);
    }
}
