package com.gic.cloud.service.impl;

import com.gic.cloud.dao.mapper.TabIndexModuleRelMapper;
import com.gic.cloud.entity.TabIndexModuleRel;
import com.gic.cloud.service.IndexModuleRelService;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author zhiwj
 * @Description:
 * @date 2020-07-08 10:26
 */
@Service
public class IndexModuleRelServiceImpl implements IndexModuleRelService {

    @Autowired
    private TabIndexModuleRelMapper tabIndexModuleRelMapper;


    @Override
    public void save(Integer businessId, List<String> moduleIdList, Integer type) {
        if (CollectionUtils.isNotEmpty(moduleIdList)) {
            // 把所有关联status设置为0
            tabIndexModuleRelMapper.delByBusinessId(businessId, type);
            // 插入不存在的关联
            for (String moduleId : moduleIdList) {
                TabIndexModuleRel rel = new TabIndexModuleRel();
                rel.setType(type);
                rel.setBusinessId(businessId);
                rel.setModuleId(moduleId);
                rel.setCreateTime(new Date());
                rel.setStatus(1);
                tabIndexModuleRelMapper.insertSelectiveByNotExist(rel);
            }
            // 把所有userId关联status都设置为1
            tabIndexModuleRelMapper.updateStatusByBusinessIds(businessId, moduleIdList, type);
        }
    }

    @Override
    public List<String> listModuleIdByBusinessId(Integer businessId, Integer type) {
        return tabIndexModuleRelMapper.listModuleIdByBusinessId(businessId, type);
    }

    @Override
    public List<Integer> listBusinessIdByModuleId(Integer moduleId) {
        return tabIndexModuleRelMapper.listBusinessIdByModuleId(moduleId);
    }

    @Override
    public void deleteByBusinessId(Integer businessId, Integer type) {
        tabIndexModuleRelMapper.delByBusinessId(businessId, type);
    }

    @Override
    public Map<Integer, List<String>> groupModuleIdByBusinessIds(List<Integer> businessIdList, Integer type) {
        List<TabIndexModuleRel> relList = tabIndexModuleRelMapper.listModuleIdByBusinessIds(businessIdList, type);
        return Optional.ofNullable(relList).orElse(Collections.emptyList()).stream()
                .collect(Collectors.groupingBy(TabIndexModuleRel::getBusinessId,
                        Collectors.mapping(TabIndexModuleRel::getModuleId, Collectors.toList())));
    }
}
