package com.gic.cloud.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.cloud.dto.DataAuthDTO;
import com.gic.cloud.dto.UserDTO;
import com.gic.cloud.entity.TabSysDataAuth;
import com.gic.cloud.entity.TabSysUser;
import com.gic.cloud.service.DataAuthApiService;
import com.gic.cloud.service.DataAuthService;
import com.gic.cloud.service.UserService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.error.ErrorCode;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

@Service("dataAuthApiService")
public class DataAuthApiServiceImpl implements DataAuthApiService{
    private static Logger LOGGER = LogManager.getLogger(DataAuthApiServiceImpl.class);

    @Autowired
    private DataAuthService dataAuthService;
    @Autowired
    private UserService userService;

    @Override
    public ServiceResponse<Integer> saveDataAuth(DataAuthDTO dto) {
        ServiceResponse validParamResult = validParam(dto);
        if (!validParamResult.isSuccess()) {
            return ServiceResponse.failure(validParamResult.getCode(), validParamResult.getMessage());
        }
        return ServiceResponse.success(dataAuthService.saveDataAuth(dto));
    }

    @Override
    public ServiceResponse<Void> editDataAuth(DataAuthDTO dto) {
        ServiceResponse validParamResult = validParam(dto);
        if (!validParamResult.isSuccess()) {
            return ServiceResponse.failure(validParamResult.getCode(), validParamResult.getMessage());
        }
        dataAuthService.editDataAuth(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<DataAuthDTO> getDataAuth(Integer dataAuthId) {
        return ServiceResponse.success(EntityUtil.changeEntityNew(DataAuthDTO.class, dataAuthService.getDataAuth(dataAuthId)));
    }

    @Override
    public ServiceResponse<Void> deleteDataAuth(Integer dataAuthId) {
        dataAuthService.deleteDataAuth(dataAuthId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Page<DataAuthDTO>> pageDataAuth(Integer enterpriseId, String dataAuthName,
                                                           Integer currentPage, Integer pageSize) {
        com.github.pagehelper.Page page = dataAuthService.pageDataAuth(enterpriseId, dataAuthName, currentPage, pageSize);
        Page<DataAuthDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, DataAuthDTO.class);
        if (resultPage != null) {
            List<DataAuthDTO> dataAuthList = resultPage.getResult();
            //查询授权人数
            if (CollectionUtils.isNotEmpty(dataAuthList)) {
                List<UserDTO> userList = userService.countGroupByDataAuthId(enterpriseId,
                        dataAuthList.stream().map(e -> e.getDataAuthId()).collect(Collectors.toList()));
                if (CollectionUtils.isNotEmpty(userList)) {
                    Map<String, Integer> map = userList.stream()
                            .collect(Collectors.toMap(e -> e.getDataAuthId().toString(), e -> e.getMemberCount()));
                    dataAuthList.forEach(e -> e.setMemberCount(map.get(e.getDataAuthId().toString())));
                }
            }
        }
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<List<DataAuthDTO>> listDataAuth(Integer enterpriseId, String dataAuthName) {
        return ServiceResponse.success(Optional.ofNullable(EntityUtil.changeEntityListNew(DataAuthDTO.class,
                dataAuthService.listDataAuth(enterpriseId, dataAuthName))).orElse(Collections.EMPTY_LIST));
    }

    @Override
    public ServiceResponse<DataAuthDTO> ggetDataAuthByUserId(Integer enterpriseId, Integer userId) {
        TabSysUser user = userService.getUser(userId);
        if (user == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户ID错误，查无数据");
        }
        return getDataAuth(user.getDataAuthId());
    }

    private ServiceResponse validParam(DataAuthDTO dto) {
        if (dto.getDataAuthId() != null) {
            TabSysDataAuth record = dataAuthService.getDataAuth(dto.getDataAuthId());
            if (record == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "数据权限集id错误，查无数据");
            }
            dto.setEnterpriseId(record.getEnterpriseId());
        }
        Integer enterpriseId = dto.getEnterpriseId();
        String dataAuthName = dto.getDataAuthName();
        if (StringUtils.isBlank(dataAuthName)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "数据权限集名称不能为空");
        }
        if (dataAuthService.isRepeatDataAuthName(enterpriseId, dto.getDataAuthId(), dataAuthName)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "数据权限集名称重复");
        }
        String memberCardAuth = dto.getMemberCardAuth();
        if (StringUtils.isNotBlank(memberCardAuth)) {
            dto.setMemberCardAuth("_" + memberCardAuth.replaceAll(",", "_") + "_");
        }
        String goodsAreaAuth = dto.getGoodsAreaAuth();
        if (StringUtils.isNotBlank(goodsAreaAuth)) {
            dto.setGoodsAreaAuth("_" + goodsAreaAuth.replaceAll(",", "_") + "_");
        }
        return ServiceResponse.success();
    }
}
