package com.gic.cloud.service.outer.impl;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.cloud.dto.IndexGroupDTO;
import com.gic.cloud.entity.TabIndexGroup;
import com.gic.cloud.service.IndexGroupApiService;
import com.gic.cloud.service.IndexGroupService;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @author zhiwj
 * @Description:
 * @date 2020-07-13 15:37
 */
@Service("indexGroupApiService")
public class IndexGroupApiServiceImpl implements IndexGroupApiService {

    @Autowired
    private IndexGroupService indexGroupService;

    @Override
    public ServiceResponse<Void> save(IndexGroupDTO indexGroupDTO) {
        // 校验参数
        if (indexGroupService.isNameRepeat(indexGroupDTO.getIndexGroupName(), null)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "名称重复");
        }
        Integer indexGroupId = indexGroupService.save(indexGroupDTO);
        indexGroupDTO.setIndexGroupId(indexGroupId);
        TabIndexGroup parentIndexGroup = indexGroupService.getById(indexGroupDTO.getParentId());
        indexGroupDTO.setGroupChain(parentIndexGroup.getGroupChain() + indexGroupId + "_");
        indexGroupService.update(indexGroupDTO);

        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> update(IndexGroupDTO indexGroupDTO) {
        // 校验参数
        if (indexGroupService.isNameRepeat(indexGroupDTO.getIndexGroupName(), null)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "名称重复");
        }

        TabIndexGroup parentIndexGroup = indexGroupService.getById(indexGroupDTO.getParentId());
        indexGroupDTO.setGroupChain(parentIndexGroup.getGroupChain() + "_" + indexGroupDTO.getIndexGroupId() + "_");
        indexGroupService.update(indexGroupDTO);

        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<IndexGroupDTO>> tree() {
        List<TabIndexGroup> groupList = this.indexGroupService.listIndexGroup();
        List<IndexGroupDTO> groupDTOList = EntityUtil.changeEntityListByJSON(IndexGroupDTO.class, groupList);
        List<IndexGroupDTO> tree = toTree(groupDTOList);
        return EnterpriseServiceResponse.success(tree);
    }

    private List<IndexGroupDTO> toTree(List<IndexGroupDTO> groupDTOList) {
        return toTree(0, groupDTOList);
    }
    private List<IndexGroupDTO> toTree(Integer parentIndexGroupId, List<IndexGroupDTO> groupDTOList) {
        if (CollectionUtils.isEmpty(groupDTOList)) {
            return null;
        }
        List<IndexGroupDTO> resultList = new ArrayList<>();
        for (IndexGroupDTO indexGroupDTO : groupDTOList) {
            if (parentIndexGroupId.equals(indexGroupDTO.getParentId())) {
                resultList.add(indexGroupDTO);
                indexGroupDTO.setChildList(toTree(indexGroupDTO.getIndexGroupId(), groupDTOList));
            }
        }
        return resultList;
    }

    @Override
    public ServiceResponse<Void> delete(Integer indexGroupId) {
        indexGroupService.delete(indexGroupId);
        return ServiceResponse.success();
    }
}
