package com.gic.cloud.web.controller;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.cloud.dto.AccountGroupDTO;
import com.gic.cloud.dto.UserDTO;
import com.gic.cloud.qo.UserQO;
import com.gic.cloud.service.*;
import com.gic.cloud.web.vo.*;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.member.config.api.dto.OwnerMemberCardDTO;
import com.gic.member.config.api.service.MemberCardApiService;

@RestController
@RequestMapping("/user")
public class UserController {
    private final static Logger LOGGER = LogManager.getLogger(UserController.class);

    @Autowired
    private AccountGroupApiService accountGroupApiService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private FunctionApiService functionApiService;
    @Autowired
    private FunctionModuleApiService functionModuleApiService;
    @Autowired
    private DataAuthApiService dataAuthApiService;
    @Autowired
    private MemberCardApiService memberCardApiService;

    @RequestMapping("/save-account-group")
    public RestResponse saveAccountGroup(AccountGroupDTO dto) {
        dto.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(accountGroupApiService.saveAccountGroup(dto));
    }

    @RequestMapping("/edit-account-group")
    public RestResponse editAccountGroup(AccountGroupDTO dto) {
        return ResultControllerUtils.commonResult(accountGroupApiService.editAccountGroup(dto));
    }

    @RequestMapping("/sort-account-group")
    public RestResponse sortAccountGroup(Integer accountGroupId, Integer targetSort) {
        return ResultControllerUtils.commonResult(accountGroupApiService.sortAccountGroup(accountGroupId, targetSort));
    }

    @RequestMapping("/list-account-group")
    public RestResponse listAccountGroup() {
        return ResultControllerUtils.commonResult(accountGroupApiService
                .listAccountGroupOfCountMember(UserDetailUtils.getUserDetail().getEnterpriseId()), AccountGroupVO.class);
    }

    @RequestMapping("/delete-account-group")
    public RestResponse deleteAccountGroup(Integer accountGroupId) {
        return ResultControllerUtils.commonResult(accountGroupApiService.deleteAccountGroup(accountGroupId));
    }

    @RequestMapping("/save-user")
    public RestResponse saveUser(UserDTO dto) {
        dto.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(userApiService.saveUser(dto));
    }

    @RequestMapping("/edit-user")
    public RestResponse editUser(UserDTO dto) {
        return ResultControllerUtils.commonResult(userApiService.editUser(dto));
    }

    @RequestMapping("/get-user-detail")
    public RestResponse getUserDetail(Integer userId) {
        return ResultControllerUtils.commonResultOne(userApiService.getUser(userId), UserDetailVO.class);
    }

    @RequestMapping("/delete-user")
    public RestResponse deleteUser(Integer userId) {
        return ResultControllerUtils.commonResult(userApiService.deleteUser(userId));
    }

    @RequestMapping("/list-function-component")
    public RestResponse listFunctionComponent(String search) {
        return ResultControllerUtils.commonResult(functionModuleApiService.listFunctionModule(search), FunctionModuleVO.class);
    }

    @RequestMapping("/list-function")
    public RestResponse listFunction(String functionName) {
        return ResultControllerUtils.commonResult(functionApiService
                .listFunction(UserDetailUtils.getUserDetail().getEnterpriseId(), functionName), FunctionListVO.class);
    }

    @RequestMapping("/list-data-auth")
    public RestResponse listDataAuth(String dataAuthName) {
        return ResultControllerUtils.commonResult(dataAuthApiService
                .listDataAuth(UserDetailUtils.getUserDetail().getEnterpriseId(), dataAuthName), DataAuthListVO.class);
    }

    @RequestMapping("/page-user")
    public RestResponse pageUser(UserQO userQO) {
        userQO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonPageResult(userApiService.pageUser(userQO), UserVO.class);
    }

    @RequestMapping("/bulk-transfer-user-to-new-account-group")
    public RestResponse bulkTransferAccountGroup(String userIds, Integer targetAccountGroupId, boolean isAll) {
        List<Integer> userIdList = null;
        if (isAll) {
            ServiceResponse<List<UserDTO>> result = userApiService.listUser(new UserQO()
                    .setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId()));
            if (result.isSuccess()) {
                List<UserDTO> userList = result.getResult();
                if (CollectionUtils.isNotEmpty(userList)) {
                    userIdList = userList.stream().map(e -> e.getUserId()).collect(Collectors.toList());
                }
            }
        } else {
            userIdList = Arrays.stream(userIds.split(","))
                    .filter(e -> StringUtils.isNumeric(e))
                    .mapToInt(e -> Integer.parseInt(e))
                    .boxed()
                    .collect(Collectors.toList());
        }
        return ResultControllerUtils.commonResult(userApiService.bulkTransferAccountGroup(userIdList, targetAccountGroupId));
    }

    @RequestMapping("/list-all-member-card")
    public RestResponse listAllMemberCard() {
        ServiceResponse<List<OwnerMemberCardDTO>> result = memberCardApiService.getAll(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(result, MemberCardVO.class);
    }
}
