package com.gic.cloud.web.auth;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.cloud.dto.DataAuthDTO;
import com.gic.cloud.service.DataAuthApiService;
import com.gic.store.service.StoreWidgetApiService;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

@Component
public class DataAuthUtils {
    private static final Logger log = LogManager.getLogger(DataAuthUtils.class);

    @Autowired
    private DataAuthApiService dataAuthApiService;
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;

    public StoreAuth getStoreAuth(Integer userId, Integer enterpriseId){
        ServiceResponse<DataAuthDTO> response = this.dataAuthApiService.ggetDataAuthByUserId(enterpriseId, userId);
        StoreAuth storeAuth = new StoreAuth();
        if(response.isSuccess()){
            storeAuth.setHasAuth(true);
            DataAuthDTO dataAuthDTO = response.getResult();
            Integer storeWidgetId = dataAuthDTO.getStoreWidgetId();
            if(storeWidgetId != null){
                List<Integer> storeInfoIdList = this.storeWidgetApiService.listStoreInfoIdByStoreWidgetId(enterpriseId, storeWidgetId).getResult();
                storeAuth.setStoreInfoIdList(storeInfoIdList);
                storeAuth.setStoreWidgetId(storeWidgetId);
            }else {
                storeAuth.setStoreInfoIdList(new ArrayList<>());
            }
        }
        return storeAuth;
    }


    public OnLineAuth getOnlineStore(Integer userId, Integer enterpriseId){
        ServiceResponse<DataAuthDTO> response = this.dataAuthApiService.ggetDataAuthByUserId(enterpriseId, userId);
        OnLineAuth onLineAuth = new OnLineAuth();
        if(response.isSuccess()){
            DataAuthDTO dataAuthDTO = response.getResult();
            String onlinePerformance = dataAuthDTO.getOnlinePerformance();
            if(StringUtils.isNotBlank(onlinePerformance)){
                onLineAuth.setHasAuth(true);
                JSONArray array = JSON.parseArray(onlinePerformance);
                List<OnLineStore> list = new ArrayList<>();
                for(int i=0; i<array.size(); i++){
                    JSONObject jsonObject = array.getJSONObject(i);
                    OnLineStore onLineStore = new OnLineStore();
                    onLineStore.setChannel(jsonObject.getInteger("channel"));
                    onLineStore.setStoreIdList(JSONArray.parseArray(jsonObject.getJSONArray("storeContent").toJSONString(), String.class));
                    list.add(onLineStore);
                }
                onLineAuth.setList(list);
            }
        }
        return onLineAuth;
    }


}
